clear
import excel inflation_data.xlsx, first

browse

tsset quarters, quarterly

//QUESTION 1

tsline CPIINFL, xtitle("") title("US Inflation")
graph export Figure1.png

*Estimate the local level (unmoored inflation expectations) model

*constraints impose appropriate forms for system matrices----------------------

//FIRST WE IMPOSE A CONSTRAINT ON THE STATE EQUATION (IE THE ALPHA EQUATION) 
//THE COEFFICIENT  ON LAGGED ALPHA IS ONE

//SECOND WE IMPOSE A CONSTRAINT ON THE MEASUREMENT EQUATION (IE THE CPIINFL EQUATION)
//THE COEFFICIENT OF ALPHA IS ONE

//SEE BOTTOM OF SLIDE 48 FOR RELATIONSHIP WITH GENERAL NORMAL LINEAR STATE SPACE MODEL

constraint 1 [alpha]L.alpha = 1 
constraint 2 [CPIINFL]alpha = 1 

*sspace actually estimates the local level model--------------------------------

//SPECIFY WHAT'S IN THE STATE EQUATION FIRST
//THEN SPECIFY WHAT'S IN THE MEASUREMENT (AKA OBSERVATION) EQUATION
//THEN IMPOSE THE CONSTRAINTS

sspace (alpha L.alpha, state noconstant) (CPIINFL alpha, noconstant), constraints(1 2)

//DISPLAY INFORMATION CRITERIA REQUIRED TO COMPARE MODELS, LOWER IS BETTER

estat ic

*The following line creates smoothed estimates of the states--------------------

///smethod(smooth) causes predict to estimate the states at each time period 
///using all the sample data by the Kalman smoother. 
predict alpha_smooth, states smethod(smooth) equation(alpha) rmse(rmse)
scalar z = invnormal(.95)
gen lb = alpha_smooth - z*(rmse)
gen ub = alpha_smooth + z*(rmse)
tsline alpha_smooth lb ub, xtitle("") title("Smoothed Alpha Estimates with 95% Confidence Interval - Local-Level")
graph export Figure2.png


*The following creates filtered estimates of the states 

///smethod(filter) causes predict to estimate the states at each time period using 
///previous and contemporaneous data by the Kalman filter. The Kalman filter is 
///performed on previous periods and the current period.
predict alpha_filt, states smethod(filter) equation(alpha) rmse(rmse1)
gen lb1 = alpha_filt - z*(rmse1)
gen ub1 = alpha_filt + z*(rmse1)
tsline alpha_filt lb1 ub1, xtitle("") title("Filtered Alpha Estimates with 95% Confidence Interval")
graph export Figure3.png


// *The following makes a graph
tsline CPIINFL alpha_smooth alpha_filt, xtitle("") title("Unmoored Inflation Expectations (Random Walk)")
graph export Figure4.png

*now do a variant of the anchored inflation expectations model------------------

//NOW WE DON'T CONSTRAIN THE COEFFICIENT ASSOCIATED WITH LAGGED ALPHA IN THE 
//STATE SPACE EQUATION TO BE ONE AND WE HAVE A CONSTANT IN THE STATE EQUATION
 
constraint 3 [CPIINFL]alpha1 = 1 
sspace (alpha1 L.alpha1, state) (CPIINFL alpha1, noconstant), constraints(3)
estat ic

*This creates smoothed estimates of the states
predict alpha_smooth1, states smethod(smooth) equation(alpha1)

*This creates filtered estimates of the states 
predict alpha_filt1, states smethod(filter) equation(alpha1)
tsline CPIINFL alpha_smooth1 alpha_filt1, xtitle("") title("Anchored Inflation Expectations")

*now plot difference between two smoothed states on same graph
generate alpha_diff = alpha_smooth1 - alpha_smooth
tsline alpha_diff, xtitle("") title("Difference Between Anchored and Unmoored")


//QUESTION 2

//THE KEY PART OF THIS QUESTION IS DISCOVERING THE NOERROR OPTION

constraint 4 [CPIINFL]u = 1
sspace (u L.u, state noconstant) (CPIINFL u, noerror), constraints(4)
estat ic
predict yhat
mean yhat

arima CPIINFL, ar(1)
estat ic
predict yhat2
mean yhat2

//QUESTION 3i

constraint 5 [beta]L.beta = 1
constraint 6 [alpha2]L.alpha2 = 1
constraint 7 [alpha2]L.beta = 1
constraint 8 [CPIINFL]alpha2 = 1

sspace (beta L.beta, state noconstant) (alpha2 L.alpha2 L.beta, state noconstant) (CPIINFL alpha2, noconstant), constraints (5 6 7 8) iter(100)

// //QUESTION 3iii
estat ic
//
// //QUESTION 3ii
predict alpha_smooth2, states smethod(smooth) equation(alpha2)
predict alpha_filt2, states smethod(filter) equation(alpha2) 
tsline CPIINFL alpha_smooth2 alpha_filt2, xtitle("") title("Anchored Inflation Expectations")



//QUESTION 3iv

//ERROR TERM WITH ZERO VARIANCE = CONSTANT SO AGAIN WE MAKE USE OF THE NOERROR OPTION

constraint 9 [beta1]L.beta1 = 1
constraint 10 [alpha3]L.alpha3 = 1
constraint 11 [alpha3]L.beta1 = 1
constraint 12 [CPIINFL]alpha3 = 1

sspace (beta1 L.beta1, state noerror) (alpha3 L.alpha3 L.beta1, state noconstant) (CPIINFL alpha3, noconstant), constraints (9 10 11 12)
estat ic

predict alpha_smooth3, states smethod(smooth) equation(alpha3) rmse(rmse2)
predict alpha_filt3, states smethod(filter) equation(alpha3) 
tsline CPIINFL alpha_smooth3 alpha_filt3, xtitle("") title("Anchored Inflation Expectations")
graph export Figure5.png

gen lb2 = alpha_smooth3 - z*(rmse2)
gen ub2 = alpha_smooth3 + z*(rmse2)
tsline alpha_smooth3 lb2 ub2, xtitle("") title("Smoothed Alpha Estimates with 95% Confidence Interval (local-Linear Trend Model)")
graph export Figure6.png
